/**
 * cfo/fo_methods/p2/factory_accu.hpp
 * @author The RECORD Development Team (INRA) and the VLE Development Team
 * See the AUTHORS or Authors.txt file
 */

/*
 * Copyright (C) 2009-2010 INRA
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CFO_FO_METHODS_P2_ACCU_FACTORY_HPP_
#define CFO_FO_METHODS_P2_ACCU_FACTORY_HPP_

#include <vle/value/Tuple.hpp>

#include <cfo/tools/accumulators/accu_multi.hpp>
#include <cfo/fo_methods/p2/accu_leaf.hpp>

namespace cfo {
namespace fo_methods {
namespace p2 {

namespace st = cfo::tools;

/**
 * @brief AccuFactory is a util class for build building LeafP2 and StratP2
 * accumulators from P2 initializations.
 */
template <class LEAF>
class AccuFactory
{
    typedef boost::shared_ptr<LEAF> leaf_type_ptr;
    typedef typename LEAF::strat_type_ptr strat_type_ptr;
    typedef typename LEAF::strat_type strat_type;

public:
    /**
     * @brief AccuFactory constructor
     * @param p2args, P2 initializations
     */
    AccuFactory(const P2args& p2args):
        mP2args(p2args)
    {
    }
    /**
     * @brief AccuFactory destructor
     */
    virtual ~AccuFactory()
    {
    }
    /**
     * @brief Builds a new LeafP2 object
     * @param depth, the depth of the leaf
     */
    leaf_type_ptr buildLeaf(unsigned int depth) const
    {
        return leaf_type_ptr(new LEAF(mP2args, depth));
    }
    /**
     * @brief Builds a new StratP2 object
     * @param input, the point
     */
    strat_type_ptr buildStrat(const vv::Tuple& input) const
    {
        return strat_type_ptr(new strat_type(input));
    }

private:
    /**
     * @brief The P2 initializations
     */
    P2args mP2args;
};

}
}
}//namespaces

#endif //CFO_FO_METHODS_P2_ACCU_FACTORY_HPP_
