/**
 * cfo/fo_methods/p2/leaf_selection.hpp
 * @author The RECORD Development Team (INRA) and the VLE Development Team
 * See the AUTHORS or Authors.txt file
 */

/*
 * Copyright (C) 2009-2010 INRA
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CFO_FO_METHODS_P2_LEAF_SELECTION_H_
#define CFO_FO_METHODS_P2_LEAF_SELECTION_H_

#include <boost/shared_ptr.hpp>
#include <deque>
#include <algorithm>

#include <cfo/fo_methods/p2/leaf_selection_alternatives.hpp>
#include <cfo/tools/accumulators/accu_multi.hpp>

namespace cfo {
namespace fo_methods {
namespace p2 {

template<class LEAF, LeafSelectionType SELECT_TYPE = LEAF_SEL_PERFORMANCE>
class LeafPool
{
public:

    typedef boost::shared_ptr<LEAF> leaf_type_ptr;
    typedef typename LEAF::strat_type strat_type;
    typedef typename std::vector<leaf_type_ptr>::iterator iterator;
    typedef typename std::vector<leaf_type_ptr>::const_iterator const_iterator;

    /**
     *@brief LeafPool constructor
     *
     */
    LeafPool(const P2args& p2args) :
        mmaxCapacity(p2args.pool_max_capacity), mleaves(), mcomp()
    {
    }
    /**
     *@brief LeafPool destructor
     *
     */
    virtual ~LeafPool()
    {

    }

    unsigned int size() const
    {
        return mleaves.size();
    }

    /**
     *@brief method for adding a leaf
     *@param toadd the leaf to store
     */
    void pushLeaf(const leaf_type_ptr& toadd)
    {
        mleaves.push_back(toadd);
        std::sort(mleaves.begin(),mleaves.end(),mcomp);
        while (mleaves.size() > mmaxCapacity) {
            mleaves.pop_back();
        }
    }

    inline leaf_type_ptr popLeaf()
    {
        leaf_type_ptr res = mleaves.front();
        mleaves.pop_front();
        return res;
    }
    /**
     *@brief selects the leaf with the minimal score1
     * (use of score2 if two leaves have the same score1)
     *@return the leaf with the minimal score
     */

private:
    unsigned int mmaxCapacity;
    std::deque<leaf_type_ptr> mleaves;
    LeafComparator<LEAF, ct::MonoDim, SELECT_TYPE> mcomp;
};

}
}
}//namespace cfo::fo_methods::p2

#endif//CFO_FO_METHODS_P2_LEAF_SELECTION_H_
