/**
 * cfo/fo_methods/p2/leaf_selection_alternatives.hpp
 * @author The RECORD Development Team (INRA) and the VLE Development Team
 * See the AUTHORS or Authors.txt file
 */

/*
 * Copyright (C) 2009-2010 INRA
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CFO_FO_METHODS_P2_LEAF_SELECTION_ALTERNATIVES_H_
#define CFO_FO_METHODS_P2_LEAF_SELECTION_ALTERNATIVES_H_

#include <vector>
#include <algorithm>

#include <boost/shared_ptr.hpp>

#include <cfo/tools/accumulators/accu_multi.hpp>
#include <cfo/fo_methods/p2/accu_leaf.hpp>
#include <cfo/fo_methods/p2/utils.hpp>



namespace cfo {
namespace fo_methods {
namespace p2 {

/**
 *@brief Object storing a set of leaves
 *
 */

namespace st = cfo::tools;

template<class LEAF, class DIM_TYPE, LeafSelectionType SEL_TYPE>
struct LeafComparator;

template<class LEAF>
struct LeafComparator<LEAF, ct::MonoDim, LEAF_SEL_PERFORMANCE>
{
    typedef boost::shared_ptr<LEAF> leaf_type_ptr;
    bool operator()(const leaf_type_ptr& i, const leaf_type_ptr& j)
    {

//        TODO : AVEC : pas de seg fault, sans : seg fault
//                i->performance();
//                j->performance();
        return (i->performance() < j->performance());
    }
};

template<class LEAF>
struct LeafComparator<LEAF, ct::MonoDim, LEAF_SEL_BETA>
{
    typedef boost::shared_ptr<LEAF> leaf_type_ptr;
    bool operator()(const leaf_type_ptr& i, const leaf_type_ptr& j)
    {
        return (score(i) < score(j));
    }

    double score(const leaf_type_ptr& leaf)
    {
        //TODO cf modif du gamma par rapport à la thèse
        double epsilon = 0.1;
        double max_depth = log(leaf->getNbPotentialSubLeaves()) + leaf->getDepth();
        double gamma = (leaf->getDepth() / (epsilon + max_depth)) * (M_PI / 2);
        double beta = leaf->dispersion() + leaf->performance() * tan(gamma);
        return beta;
    }
};

template<class LEAF>
struct LeafComparator<LEAF, ct::MonoDim, LEAF_SEL_CONC_BETA>
{
    typedef boost::shared_ptr<LEAF> leaf_type_ptr;
    bool operator()(const leaf_type_ptr& i, const leaf_type_ptr& j)
    {
        return (score(i) < score(j));
    }

    double score(const leaf_type_ptr& leaf)
    {
        //TODO cf modif du gamma par rapport à la thèse
        double epsilon = 0.1;
        double max_depth = log(leaf->getNbPotentialSubLeaves()) + leaf->getDepth();
        double gamma = (leaf->getDepth() / (epsilon + max_depth)) * (M_PI / 2);
        double projPerf = leaf->performance() * cos((M_PI / 2) - gamma);
        double projDisp = leaf->dispersion() * sin((M_PI / 2) - gamma);
        return std::min(projDisp,projPerf);
    }
};

}
}
}//namespace cfo::fo_methods::p2

#endif//CFO_FO_METHODS_P2_LEAF_SELECTION_ALTERNATIVES_H_
