/**
 * cfo/fo_methods/shark/utils.hpp
 * @author The RECORD Development Team (INRA) and the VLE Development Team
 * See the AUTHORS or Authors.txt file
 */

/*
 * Copyright (C) 2009-2010 INRA
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CFO_FO_METHODS_SHARK_UTILS_HPP_
#define CFO_FO_METHODS_SHARK_UTILS_HPP_


#include <EALib/ChromosomeCMA.h>

namespace cfo {
namespace fo_methods {
namespace shark {



/**
 * @brief Global structure that stores sharkCMA parameters.
 */
struct SharkCMAargs
{
    /**
     * @brief SharkCMA default constructor
     */
    SharkCMAargs() :
        minInit(0), maxInit(0),
        nb_iterations(100),
        global_step_init(20.0),
        sim_common_seed(false),
        sim_nb_replicas(1),
        sim_output_scale(1)
    {
    }
    /**
     * @brief SharkCMA copy constructor
     */
    SharkCMAargs(const SharkCMAargs& args) :
        minInit(args.minInit), maxInit(args.maxInit),
        nb_iterations(args.nb_iterations),
        global_step_init(args.global_step_init),
        sim_common_seed(args.sim_common_seed),
        sim_nb_replicas(args.sim_nb_replicas),
        sim_output_scale(args.sim_output_scale)
    {
    }
    /**
     * @brief Chromosome with minimal values for each parameter
     */
    ChromosomeT<double> minInit;
    /**
     * @brief Chromosome with maximal values  for each parameter
     */
    ChromosomeT<double> maxInit;

    /**
     * @brief Number of iterations of the CMA ES.
     */
    unsigned int nb_iterations;
    /**
     * @brief Initial global step size
     */
    double global_step_init;
    /**
     * @brief Common seed option
     */
    bool sim_common_seed;
    /**
     * @brief number of replicas for the simulation of one point
     */
    int sim_nb_replicas;
    /**
     * @brief Scale for the simulation output
     * (eg. -1 for maximisation)
     */
    double sim_output_scale;

};

}
}
}

#endif /* CFO_FO_METHODS_SHARK_UTILS_HPP_ */
