/**
 * cfo/interfaces/FOexpe.hpp
 * @author The RECORD Development Team (INRA) and the VLE Development Team
 * See the AUTHORS or Authors.txt file
 */

/*
 * Copyright (C) 2009-2010 INRA
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CFO_INTERFACES_FO_EXPE_HPP_
#define CFO_INTERFACES_FO_EXPE_HPP_


#include <vle/devs/Dynamics.hpp>
#include <vle/value/Value.hpp>

#include <cfo/tools/VleSimulator.hpp>
#include <cfo/interfaces/AbstractExpe.hpp>


#include <iostream>

namespace cfo {
namespace interfaces {

namespace ct = cfo::tools;
namespace vv = vle::value;
namespace vd = vle::devs;
namespace vu = vle::utils;

/**
 * @brief Instantiation of an experiment for an function optimization
 * problem. Template FO_METHOD is the instantiation of the function
 * optimization method (FOmethod)
 */
template<class FO_METHOD>
class FOexpe : public AbstractExpe
{
public:
    /**
     * @brief FOexpe constructor
     */
    FOexpe(const vd::DynamicsInit& init, const vd::InitEventList& events) :
        AbstractExpe(init,events), mmethod(0)
    {
    }
    /**
     * @brief FOexpe destructor
     */
    virtual ~FOexpe()
    {
       if(mmethod){
           delete mmethod;
       }
    }
    /**
     * @brief Method for initializing the experiment
     */
    virtual void initExpe()
    {
        if(mmethod){
            delete mmethod;
        }
        mmethod = new FO_METHOD(getInitializations(), getSimulator());
    }
    /**
     * @brief Method for a process step
     * @param current_step the current step
     * @return true if the process should stop
     */
    virtual bool processStep(const vv::Integer& current_step)
    {
    	return mmethod->optimizeStep(current_step.value());
    }

    /**
     * @brief Method that observes ports
     * @param obs, the port to observe
     * @return the observed value
     */
    virtual vv::Value* observe(const std::string& port) const
	{
        if(mmethod){
            return mmethod->observe(port);
        }
        return 0;
	}
private:
    /**
     * @brief the function optimization method
     */
    FO_METHOD* mmethod;
};


}}// namespaces

#endif
