/**
 * cfo/interfaces/RLexpe.hpp
 * @author The RECORD Development Team (INRA) and the VLE Development Team
 * See the AUTHORS or Authors.txt file
 */

/*
 * Copyright (C) 2009-2010 INRA
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CFO_INTERFACES_RL_EXPE_HPP_
#define CFO_INTERFACES_RL_EXPE_HPP_


#include <vle/devs/Dynamics.hpp>
#include <vle/value/Value.hpp>

#include <cfo/tools/VleSimulator.hpp>
#include <cfo/interfaces/AbstractExpe.hpp>


#include <iostream>

namespace cfo {
namespace interfaces {

/**
 * @brief Instantiation of an experiment for reinforcement learning problem.
 */
template <class RL_EXPE_METHOD>
class RLexpe : public AbstractExpe
{
    RL_EXPE_METHOD* rlemethod;

public:
    /**
     * @brief RLexpe constructor
     */
    RLexpe(const vd::DynamicsInit& init, const vd::InitEventList& events) :
        AbstractExpe(init,events), rlemethod(0)
    {
    }


    /**
     * @brief Method for initializing the experiment
     */
    virtual void initExpe()
    {
        rlemethod = new RL_EXPE_METHOD(getInitializations(), getSimulator());
        if(getInitializations().exist("parameters")){
            getSimulator().setCondValue("CallForOptim","parameters",
                    getInitializations().getMap("parameters"));
        }
        if(getInitializations().exist("observed")){
            getSimulator().setCondValue("CallForOptim","observed",
                    getInitializations().getSet("observed"));
        }
        rlemethod->initExpe();
    }

    /**
     * @brief RLexpe destructor
     */
    virtual ~RLexpe()
    {
        delete rlemethod;
    }

    /**
     * @brief Method for a process step
     * @param current_step the current step
     * @return true if the process should stop
     */
    virtual bool processStep(const vv::Integer& current_step)
    {
    	return rlemethod->processStep(current_step);
    }

    /**
     * @brief Method that observes one port
     * @param obs, the port observed
     * @return the observed value
     */
    virtual vv::Value* observe(const std::string& port) const
	{
        return rlemethod->observe(port);
	}

};

}}// namespaces


#endif
