/**
 * cfo/interfaces/STATmethod.hpp
 * @author The RECORD Development Team (INRA) and the VLE Development Team
 * See the AUTHORS or Authors.txt file
 */

/*
 * Copyright (C) 2009-2010 INRA
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CFO_INTERFACES_STATmethod_HPP_
#define CFO_INTERFACES_STATmethod_HPP_

#include <vector>
#include <utility>
#include <cmath>
#include <iostream>

#include <vle/value/Map.hpp>
#include <vle/utils/Rand.hpp>
#include <vle/devs/Dynamics.hpp>
#include <vle/devs/DynamicsDbg.hpp>

#include <cfo/tools/VleSimulator.hpp>
#include <cfo/interfaces/STATexpe.hpp>

namespace cfo {
namespace interfaces {

namespace vu = vle::utils;
namespace vv = vle::value;
namespace ct = cfo::tools;

/**
 * @brief Class that has to be implemented by all
 * statistics methods
 */
class STATmethod
{
public:
    /**
     * @brief STATmethod constructor
     * @param init a map that contains initialization parameters for the
     * statistics method, note that the same Map is used to initialized the
     * simulator.
     * @param simulator a reference to the simulator
     */
    STATmethod(const vv::Map& init, ct::VleSimulator& simulator):
        minit(init), msimulator(simulator)
    {
    }
    /**
     * @brief STATmethod destructor
     */
    virtual ~STATmethod()
    {
    }
    /**
     * @brief Function to implement by a statistic method.
     */
    virtual void make_stats() = 0;
    /**
     * @brief Observation method
     * @param port, the name of the port to observe
     * @return the observed value
     */
    virtual vv::Value* observe(const std::string& /*port*/) const
    {
    	return 0;
	}
    /**
     * @brief Access method to the simulator
     * return a reference to the simulator
     */
    ct::VleSimulator& simulator()
    {
        return msimulator;
    }

protected:
    /**
     * @brief A reference on the initialization object
     */
    const vv::Map& minit;
    /**
     * @brief A reference on the the simulator
     */
    ct::VleSimulator& msimulator;
};

}}//namespaces

#define DECLARE_STAT_METHOD(method)\
    DECLARE_DYNAMICS(cfo::interfaces::STATexpe<method >)

#endif
