/**
 * @file cfo/oov/policies/xitekPolicyCMACoov.cpp
 * @author The RECORD Development Team (INRA)
 */

/*
 * Copyright (C) 2009 INRA
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <iostream>
#include <fstream>

#include <vle/oov/Plugin.hpp>
#include <vle/utils/i18n.hpp>

#include <cfo/rl_methods/xitek/xitek_agent.hpp>
#include <cfo/tools/regression/CMAC.hpp>
#include <cfo/tools/doe/full_factorial.hpp>

namespace cfo {
namespace oov {
namespace policies {

namespace cxi = cfo::rl_methods::xitek;
namespace ctr = cfo::tools::regression;
namespace ctd = cfo::tools::doe;

class xitekPolicyCMACoov : public vle::oov::Plugin
{
public:
    xitekPolicyCMACoov(const std::string& location)
        : vle::oov::Plugin(location), mfileName(location), mfile()
    {
    }

    virtual ~xitekPolicyCMACoov()
    {
    }

    virtual void onParameter(const std::string& /*plugin*/,
                             const std::string& /*location*/,
                             const std::string& /*file*/,
                             vle::value::Value* parameters,
                             const double& /*time*/)
    {
        delete parameters;
    }

    virtual void onNewObservable(const std::string& /*simulator*/,
                                 const std::string& /*parent*/,
                                 const std::string& /*port*/,
                                 const std::string& /*view*/,
                                 const double& /*time*/)
    {
    }

    virtual void onDelObservable(const std::string& /*simulator*/,
                                 const std::string& /*parent*/,
                                 const std::string& /*port*/,
                                 const std::string& /*view*/,
                                 const double& /*time*/)
    {
    }

    virtual void onValue(const std::string& /*simulator*/,
                         const std::string& /*parent*/,
                         const std::string& /*port*/,
                         const std::string& /*view*/,
                         const double& /*time*/,
                         vle::value::Value* value)
    {
        if (value){
            mfile.open(mfileName.c_str());
            bool evalPol = false;//TODO should be parameter
            if (evalPol) {
                mfile << "stage;observables;bestAction;meanNbCall" << std::endl;
                cxi::xitekPolicy<ctr::CMAC> pol;
                pol.fromValue(*value);
                cxi::xitekPolicy<ctr::CMAC>::stage_container_it itb = pol.stages().begin();
                cxi::xitekPolicy<ctr::CMAC>::stage_container_it ite = pol.stages().end();
                for(;itb!=ite;itb++){//for each stage
                    const std::string& stage_name = itb->first;
                    cxi::xitekStage<ctr::CMAC>& stage = itb->second;
                    //gets the bounds from the first regressor
                    if(stage.nbActions() == 0){
                        throw vu::ArgError("[xitekPolicyCMACoov] missing actions");
                    }
                    const ctr::CMAC& cmac = stage.getRegressor(0);
                    const vv::Set& bounds = cmac.bounds();
                    //build the levels for the full factorial doe
                    std::vector<unsigned int> levels(
                            cmac.getDimension(),
                            cmac.nbTiles() * 3);
                    //build the full factorial doe
                    ctd::FullFactorial ff(levels,bounds);
                    //predict the best action and the mean nb of calls for all point
                    for(unsigned int p=0; p<ff.size();p++){
                        const vv::Tuple& pt = ff.point(p);
                        unsigned int bestAct = stage.getBestAction(pt);
                        double meanNbCall = 0;
                        for(unsigned int a = 0; a < stage.nbActions();a++){
                            meanNbCall += stage.getRegressor(a).getMeanNbCall(pt);
                        }
                        mfile << stage_name << ";" << pt  << ";" << bestAct << ";"
                                << meanNbCall << std::endl;
                    }
                }
            } else {
                value->writeXml(mfile);
            }
            mfile.close();
            delete value;
        }
    }

    virtual void close(const double& /*time*/)
    {

    }
private:
    std::string mfileName;
    std::ofstream   mfile;
};

}}} // namespaces

DECLARE_OOV_PLUGIN(cfo::oov::policies::xitekPolicyCMACoov)
