/**
 * @file cfo/tools/data/ReadData.hpp
 * @author The RECORD Development Team (INRA)
 * See the AUTHORS or Authors.txt file
 */

/*
 * Copyright (C) 2009-2011 INRA
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */



#ifndef CFO_TOOLS_DATA_READDATA_HPP_
#define CFO_TOOLS_DATA_READDATA_HPP_

#include <iostream>
#include <vector>

#include <boost/algorithm/string.hpp>
#include <boost/lexical_cast.hpp>

#include <cfo/tools/data/Data.hpp>

namespace cfo {
namespace tools {
namespace data {

/**
 * @brief Class that reads a Data structure from a file.
 */
class ReadData
{
public:
    /**
     * @brief ReadData constructor
     * @param filename, the name of the file
     * @param headers, the name of the headers to attach to the data
     */
    ReadData(std::istream& stream, std::vector<std::string> headers):
        mdata()
    {

        std::string line;
        std::vector<std::string> lineValues;
        std::vector<std::string> header;
        std::vector<double> doubleValues;
        if(stream.eof()){
            throw vle::utils::FileError(boost::format(_
               ("ReadData : empty file")));
        }
        //read headers
        unsigned int nbCols = 0;
        for(unsigned int i=0; i < headers.size() ; i++){
            lineValues.clear();
            header.clear();
            getline(stream,line);
            boost::split(lineValues, line, boost::is_any_of(";"));
            if(i == 0){
                nbCols = lineValues.size();
                mdata.setCols(nbCols);
            } else {
                if(lineValues.size() != nbCols){
                    throw vle::utils::FileError(boost::format(_
                        ("ReadData : unexpected header size")));
                }
            }
            Header& h = mdata.addHeader(headers[i]);
            for(unsigned int j = 0; j < lineValues.size(); j++){
                if(lineValues[j] != ""){
                    h.addProperty(j,lineValues[j]);
                }
            }
        }
        //data
        while(not stream.eof()){
            lineValues.clear();
            doubleValues.clear();
            getline(stream,line);
            if(not stream.eof()) {
                boost::split(lineValues, line, boost::is_any_of(";"));
                for(unsigned int i=0; i < lineValues.size(); i++){
                    doubleValues.push_back(
                        boost::lexical_cast <double> (lineValues[i]));
                }
                mdata.addLine(doubleValues);
            }
        }
    }

    /**
     * @brief Constant access to the data
     * @return a reference on the data
     */
    const Data& getData() const
    {
        return mdata;
    }

private:
    /**
     *@brief The data built from the file
     */
    Data mdata;

};
}}} //namespaces

#endif
