/**
 * cfo/tools/regression/BatchRegressor.hpp
 * @author The RECORD Development Team (INRA) and the VLE Development Team
 * See the AUTHORS or Authors.txt file
 */

/*
 * Copyright (C) 2009-2010 INRA
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef CFO_TOOLS_REGRESSION_BATCH_REGRESSOR_HPP_
#define CFO_TOOLS_REGRESSION_BATCH_REGRESSOR_HPP_

#include <vector>
#include <numeric>

#include <boost/multi_array.hpp>

#include <vle/value/Tuple.hpp>
#include <vle/utils/Rand.hpp>

#include <cfo/tools/data/Data.hpp>


namespace cfo {
namespace tools {
namespace regression {

namespace vv = vle::value;
namespace vu = vle::utils;
namespace ct = cfo::tools;
namespace ctd = cfo::tools::data;

/**
 * @brief BatchRegressor tools interface
 */
class BatchRegressor
{
public:
    /**
     * @brief BatchRegressor constructor
     */
    BatchRegressor()
    {
    }
    /**
     * @brief Initialization of the regressor
     * @param parameters the parameters of the method
     */
    virtual void init(const vv::Value& /*parameters*/) = 0;
    /**
     * @brief Learn the regression from data
     * @param data, the data to learn the model
     */
    virtual void learn(const ctd::Data& data) = 0 ;
    /**
     * @brief Predicts the output value at a point
     * @param point the input value
     * @return the predicted value
     */
    virtual double predict(const vv::Tuple& point) = 0;
    /**
     * @brief Converts the regressor and its parameters into a value
     * @return the value that represents the regressor
     */
    virtual vv::Value* toValue() const
    {
        return 0;
    }
    /**
     * @brief Builds the regressor and its parameters from a value
     * @param the value that represents the regressor
     */
    virtual void fromValue(const vv::Value&)
    {
    }


};

}}}//namespaces

#endif /* CFO_TOOLS_REGRESSION_BATCH_REGRESSOR_HPP_ */

