/**
 * cfo/tools/regression/IncrRegressor.hpp
 * @author The RECORD Development Team (INRA) and the VLE Development Team
 * See the AUTHORS or Authors.txt file
 */

/*
 * Copyright (C) 2009-2010 INRA
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef CFO_TOOLS_REGRESSION_INCR_REGRESSOR_HPP_
#define CFO_TOOLS_REGRESSION_INCR_REGRESSOR_HPP_

#include <vector>
#include <numeric>

#include <boost/multi_array.hpp>

#include <vle/value/Tuple.hpp>

namespace cfo {
namespace tools {
namespace regression {

namespace vv = vle::value;
namespace ct = cfo::tools;

/**
 * @brief IncrRegressor tools interface
 */
class IncrRegressor
{
public:
    /**
     * @brief IncrRegressor constructor
     */
    IncrRegressor()
    {
    }

    /**
     * @brief IncrRegressor destructor
     */
    virtual ~IncrRegressor()
    {
    }


    /**
     * @brief Initialization of the regressor
     * @param parameters the parameters of the method
     */
    virtual void init(const vv::Value& /*parameters*/) = 0;

    /**
     * @brief Predicts the output value at a point
     * @param point the input value
     * @return the predicted value
     */
    virtual double predict(const vv::Tuple& point) = 0;

    /**
     * @brief Updates the parameters of the regressor
     * according an initial update rate parameter
     * @param point the point considered
     * @param upValue the observed value at point point
     * @param alpha0 an initial update rate parameter
     * @return true if the update is a success
     */
    virtual bool updateWeight(const vv::Tuple& point,
        double upValue, double alpha0) = 0;
    /**
     * @brief Converts the Incremental regressor and its parameters into a value
     * @return the value that represents the regressor
     */
    virtual vv::Value* toValue() const
    {
        return 0;
    }
    /**
     * @brief Builds the Incremental regressor and its parameters from a value
     * @param the value that represents the regressor
     */
    virtual void fromValue(const vv::Value&)
    {
    }


};

}}}//namespaces

#endif /* CFO_TOOLS_REGRESSION_INCR_REGRESSOR_HPP_ */

