/**
 * cfo/examples/ExObserver.cpp
 * @author The RECORD Development Team (INRA) and the VLE Development Team
 * See the AUTHORS or Authors.txt file
 */

/*
 * Copyright (C) 2009-2010 INRA
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <vle/devs/DynamicsDbg.hpp>
#include <vle/devs/Dynamics.hpp>
#include <vle/value/Map.hpp>
#include <vle/utils/Exception.hpp>
#include <iostream>
#include <sstream>
#include <vector>

namespace cfo {
namespace examples {

namespace vd = vle::devs;
namespace vv = vle::value;
namespace vu = vle::utils;

/**
 * @brief Model that wake up, just once,  another model by sending
 * an optimization request
 */
class ExObserver: public vd::Dynamics
{
public:
    ExObserver(const vd::DynamicsInit& init, const vd::InitEventList& events) :
        vd::Dynamics(init,events), mresults(0)
    {
    }

    virtual ~ExObserver()
    {
        if(mresults){
            delete mresults;
        }
    }
    /**
     * @brief Implementation of Dynamics::init
     */
    virtual vd::Time init(const vd::Time& /* time */)
    {
        return vd::infinity;
    }
    /**
     * @brief Implementation of Dynamics::timeAdvance
     */
    virtual vd::Time timeAdvance() const
    {
        return vd::infinity;
    }
    /**
     * @brief Implementation of Dynamics::externalTransition
     */
    void externalTransition(const vd::ExternalEventList& event,
        const vd::Time& /* time */)
    {
        vd::ExternalEventList::const_iterator itb = event.begin();
        vd::ExternalEventList::const_iterator ite = event.end();
        for(;itb!=ite;itb++){
            if ((*itb)->onPort("expeRes")) {
                setResults((*itb)->getAttributes());
            }
        }
    }
    /**
     * @brief Implementation of Dynamics::observation
     */
    vv::Value* observation(const vd::ObservationEvent& event) const
    {
        if(mresults){
            if(mresults->exist(event.getPortName())){
                return mresults->get(event.getPortName())->clone();
            }
        }
        return 0;
    }

private:
    /**
     * @brief the last results
     */
    vv::Map* mresults;
    /**
     * @brief Store a copy the results
     * @param res, the map of results
     */
    void setResults(const vv::Map& res)
    {
        if(mresults){
            delete mresults;
        }
        mresults = new vv::Map(res);
    }
    /**
     * @brief Gets the results
     * @return the map of results
     */
    const vv::Map& getResults() const
    {
        if(!mresults){
            throw vu::InternalError(vle::fmt(_(
                " [%1%] : no results "))
                % getModelName());
        }
        return *mresults;
    }
};

}
} // namespace

DECLARE_DYNAMICS_DBG(cfo::examples::ExObserver)
