/**
 * cfo/examples/ExWakeUp.cpp
 * @author The RECORD Development Team (INRA) and the VLE Development Team
 * See the AUTHORS or Authors.txt file
 */

/*
 * Copyright (C) 2009-2010 INRA
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <vle/devs/DynamicsDbg.hpp>
#include <vle/devs/Dynamics.hpp>
#include <vle/value/Map.hpp>

namespace cfo {
namespace examples {

namespace vd = vle::devs;
namespace vv = vle::value;

/**
 * @brief Model that wake up, just once,  another model by sending
 * an optimization request
 */
class ExWakeUp: public vd::Dynamics
{
private:
    enum state
    {
        wait, wakeUp
    };
    state mstate;
    double mtimeToWakeUp;
    vv::Map minitalisations;
public:
    ExWakeUp(const vd::DynamicsInit& init, const vd::InitEventList& events) :
        vd::Dynamics(init,events), mstate(wakeUp), mtimeToWakeUp(3),
        minitalisations(events)
    {
        if(events.exist("timeToWakeUp")){
            mtimeToWakeUp = events.getDouble("timeToWakeUp");
        }
    }

    virtual ~ExWakeUp()
    {
    }

    virtual vd::Time init(const vd::Time& /* time */)
    {
        mstate = wakeUp;
        return timeAdvance();
    }

    virtual vd::Time timeAdvance() const
    {
        switch (mstate){
        case wakeUp :
            return mtimeToWakeUp;
            break;
        case wait:
            return vd::infinity;
            break;
        }
        return 0;
    }

    virtual void internalTransition(const vd::Time& /* time */)
    {
        if (mstate == wakeUp) {
            mstate = wait;
        }
    }
    virtual void output(const vd::Time& /* time */, vd::ExternalEventList& output) const
    {
        if (mstate == wakeUp) {
            vd::ExternalEvent* ee = new vd::ExternalEvent("expeReq");
            ee->putAttributes(minitalisations);
            output.push_back(ee);
        }
    }
};

}
} // namespace

DECLARE_DYNAMICS_DBG(cfo::examples::ExWakeUp)
