/**
 * @file test/cfo/fo_methods/shark/tests_shark.cpp
 * @author The RECORD Development Team (INRA) and the VLE Development Team
 * See the AUTHORS or Authors.txt file
 */

/*
 * Copyright (C) 2009-2010 INRA
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#define BOOST_TEST_MAIN
#define BOOST_AUTO_TEST_MAIN
#define BOOST_TEST_DYN_LINK
#define BOOST_TEST_MODULE tests_cfo
#include <boost/test/unit_test.hpp>
#include <boost/test/auto_unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>
#include <boost/lexical_cast.hpp>
#include <stdexcept>
#include <limits>
#include <fstream>
#include <iostream>
#include <vector>

#include <cfo/tools.hpp>

#include <vle/vpz.hpp>
#include <vle/manager.hpp>
#include <vle/oov.hpp>
#include <vle/value.hpp>


namespace ct = cfo::tools;
namespace vz = vle::vpz;
namespace vm = vle::manager;
namespace vo = vle::oov;
namespace vv = vle::value;

struct F
{
    F()
    {
        vle::manager::init();
    }

    ~F()
    {
        vle::manager::finalize();
    }

};
BOOST_GLOBAL_FIXTURE(F)

//Optimisation
BOOST_AUTO_TEST_CASE(test_optim_random)
{
    //build the vpz optimizer
    vz::Vpz vpz(vu::Path::path().getExternalPackageExpFile("CallForOptim",
        "UnitTestMethods.vpz"));
    //set conditions "mod" and "sharkCMA" to the optimizer
    std::vector<std::string> conds;
    conds.push_back("FOsim");
    conds.push_back("sharkCMA");
    conds.push_back("mode");
    ct::VleAPIfacilities::attachConditions(vpz,"Experiments",conds);
    //set the dynamic FOmethod_random to the experiment
    ct::VleAPIfacilities::setDynamics(vpz,"Experiments","dFOsharkCMA");
    //set the view of the experiment to storage plugin
    ct::VleAPIfacilities::changePlugin(vpz,"view","storage");
    //launch optimization
    vm::RunVerbose jrm(std::cout);
    jrm.start(vpz);
    //recover optimiser results
    const vv::Value* val = ct::VleAPIfacilities::findLastOutputValue(
        jrm.outputs(), "view", "UnitTestMethods:Experiments", "y-min");
    double valDouble = val->toDouble().value();
    //check optimized value
    BOOST_REQUIRE_CLOSE(valDouble,0.627525,10e-4);
}

