/**
 * @file test/cfo/models/tests_models.cpp
 * @author The RECORD Development Team (INRA) and the VLE Development Team
 * See the AUTHORS or Authors.txt file
 */

/*
 * Copyright (C) 2009-2010 INRA
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#define BOOST_TEST_MAIN
#define BOOST_AUTO_TEST_MAIN
#define BOOST_TEST_DYN_LINK
#define BOOST_TEST_MODULE tests_cfo
#include <boost/test/unit_test.hpp>
#include <boost/test/auto_unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>
#include <boost/lexical_cast.hpp>
#include <stdexcept>
#include <limits>
#include <fstream>
#include <iostream>
#include <vector>

#include <cfo/tools.hpp>

#include <vle/vpz/Vpz.hpp>
#include <vle/manager/Simulation.hpp>
#include <vle/utils/Package.hpp>
#include <vle/value/Double.hpp>


namespace ct = cfo::tools;
namespace ctp = cfo::tools::policies;
namespace ctr = cfo::tools::regression;
namespace vv = vle::value;
namespace vu = vle::utils;
namespace vz = vle::vpz;
namespace vm = vle::manager;


//Test "on request" behavior of an experiment with simple stat method
BOOST_AUTO_TEST_CASE(test_expe_on_request)
{
    //build the vpz
    vu::Package pkg("CallForOptim");
    vz::Vpz vpz(pkg.getExpFile("UnitTestMode.vpz", vu::PKG_BINARY));
    //set the view of the experiment to storage plugin
    ct::VleAPIfacilities::changePlugin(vpz,"view","storage");
    //launch optimization
    vm::Error error;
    vu::ModuleManager modules;
    vm::Simulation sim(vm::LOG_NONE, vm::SIMULATION_NONE, NULL);
    vv::Map* res = sim.run(new vz::Vpz(vpz), modules, &error);
    bool simOk = (error.code ==0);
    BOOST_REQUIRE(simOk);

    //recover mean results
    const vv::Value* val = ct::VleAPIfacilities::findLastOutputValue(
        *res, "view", "UnitTestMode:ExObserver", "mean");
    double valDouble = val->toDouble().value();
    //check mean value
    BOOST_REQUIRE_CLOSE(valDouble,6745.2979730684065,10e-4);
    //recover stdDeviation results
    val = ct::VleAPIfacilities::findLastOutputValue(
        *res, "view", "UnitTestMode:Experiments", "stdDeviation");
    valDouble = val->toDouble().value();
    //check stdDeviation value
    BOOST_REQUIRE_CLOSE(valDouble,3628.4442121079337,10e-4);
    delete res;
}
