/**
 * @file test/cfo/rl_methods/sarsa/tests_rl_sarsa.cpp
 * @author The RECORD Development Team (INRA) and the VLE Development Team
 * See the AUTHORS or Authors.txt file
 */

/*
 * Copyright (C) 2009-2010 INRA
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#define BOOST_TEST_MAIN
#define BOOST_AUTO_TEST_MAIN
#define BOOST_TEST_DYN_LINK
#define BOOST_TEST_MODULE tests_cfo
#include <boost/test/unit_test.hpp>
#include <boost/test/auto_unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>
#include <boost/lexical_cast.hpp>
#include <stdexcept>
#include <limits>
#include <fstream>
#include <iostream>


#include <vle/value/Set.hpp>
#include <vle/vpz/Vpz.hpp>
#include <vle/utils/Package.hpp>


#include <cfo/tools.hpp>
//#include <cfo/rl_methods/sarsa/sarsa_agent.hpp>

namespace ct = cfo::tools;
namespace vv = vle::value;
namespace vz = vle::vpz;
namespace vu = vle::utils;
namespace vm = vle::manager;
//namespace cs = cfo::rl_methods::sarsa;


namespace cfo{

std::string bestActionInMinesState(const vv::Set& qmatrix,
    unsigned int x, unsigned int y)
{
    vv::Set::const_iterator itb = qmatrix.begin();
    vv::Set::const_iterator ite = qmatrix.end();
    int best_act = 4;
    std::string best_act_str;
    for(;itb!=ite;itb++){
        if(((*itb)->toMap().getMap("state").getInt("x") == (int) x)&&
                ((*itb)->toMap().getMap("state").getInt("y") == (int) y)){
            best_act = (*itb)->toMap().getInt("best-action");
        }
    }

    switch(best_act){
    case 0:
        return std::string("^");
        break;
    case 1:
        return std::string(">");
        break;
    case 2:
        return std::string("v");
        break;
    case 3:
        return std::string("<");
        break;
    default:
        return std::string("*");
        break;
    }


}

}//namespace

BOOST_AUTO_TEST_CASE(test_rl_sarsa)
{
    //build the vpz optimizer
    vu::Package pkg("CallForOptim");
    vz::Vpz vpz(pkg.getExpFile("UnitTestMethods.vpz"));
    //set conditions "RLmod" and "sarsa" to the optimizer
    std::vector<std::string> conds;
    conds.push_back("RLsim");
    conds.push_back("sarsa");
    conds.push_back("mode");
    ct::VleAPIfacilities::attachConditions(vpz,"Experiments",conds);
    //set the dynamic FOmethod_random to the experiment
    ct::VleAPIfacilities::setDynamics(vpz,"Experiments","dRLexpe");
    //set the view of the optimizer to storage plugin
    ct::VleAPIfacilities::changePlugin(vpz,"view","storage");
    //launch optimization
    vu::ModuleManager modules;
    vm::Simulation simpleSimulator(vm::LOG_NONE, vm::SIMULATION_NONE, NULL);
    vm::Error error;
    vv::Map* res = simpleSimulator.run(new vz::Vpz(vpz), modules, &error);
    bool simOk = (error.code ==0);
    BOOST_REQUIRE(simOk);
    //recover optimization results
    const vv::Value* val = ct::VleAPIfacilities::findLastOutputValue(
        *res, "view", "UnitTestMethods:Experiments", "qmatrix");
    const vv::Set& qmat = val->toSet();

    //Print the best policy on the grid
    for(unsigned int x = 0; x < 6; x++){
        for(unsigned int y = 0; y < 18; y++){
            std::cout << cfo::bestActionInMinesState(qmat, x, y) << " ";
        }
        std::cout << std::endl;
    }
    //end print
    std::string bestact = cfo::bestActionInMinesState(qmat,4,9); //just on left of goal
    BOOST_REQUIRE_EQUAL(bestact,">");
    std::string bestact2 = cfo::bestActionInMinesState(qmat,4,11); //just on right of goal
    BOOST_REQUIRE_EQUAL(bestact2,"<");
}
