
/**
 * @file test/cfo/tools/data/tests_data.cpp
 * @author The RECORD Development Team (INRA) and the VLE Development Team
 * See the AUTHORS or Authors.txt file
 */

/*
 * Copyright (C) 2009-2010 INRA
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#define BOOST_TEST_MAIN
#define BOOST_AUTO_TEST_MAIN
#define BOOST_TEST_DYN_LINK
#define BOOST_TEST_MODULE tests_data

#include <stdexcept>
#include <limits>
#include <fstream>
#include <iostream>

#include <boost/test/unit_test.hpp>
#include <boost/test/auto_unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <cfo/tools/accumulators/accu_mono.hpp>
#include <cfo/tools/accumulators/accu_multi.hpp>

//Accumulators
BOOST_AUTO_TEST_CASE(test_accumulators)
{
    namespace ct = cfo::tools;
    {
        ct::Accu<ct::MonoDim, ct::MEAN_VAR> acc;
        acc.insert(1);
        acc.insert(5);
        acc.insert(4);
        acc.insert(3.6);
        BOOST_REQUIRE_CLOSE(acc.mean(),3.4,10e-4);
        BOOST_REQUIRE_CLOSE(acc.moment2(),13.74,10e-4);
        ct::Accu<ct::MonoDim, ct::MEAN_VAR> acc1;
        acc1.insert(8);
        acc1.insert(3);
        acc1.insert(2);
        acc.insertAcc(acc1);
        BOOST_REQUIRE_CLOSE(acc.mean(),3.8,10e-4);
        BOOST_REQUIRE_CLOSE(acc.moment2(),18.85143,10e-4);
    }
    {
        ct::Accu<ct::MonoDim, ct::QUANTILE> acc;
        acc.insert(1);
        acc.insert(5);
        acc.insert(4);
        acc.insert(3.6);
        BOOST_REQUIRE_CLOSE(acc.mean(),3.4,10e-4);
        BOOST_REQUIRE_CLOSE(acc.squareSum(),54.96,10e-4);
        ct::Accu<ct::MonoDim, ct::QUANTILE> acc1;
        acc1.insert(8);
        acc1.insert(3);
        acc1.insert(2);
        acc.insertAcc(acc1);
        BOOST_REQUIRE_CLOSE(acc.mean(),3.8,10e-4);
        BOOST_REQUIRE_CLOSE(acc.squareSum(),131.96,10e-4);
    }
}
