
/**
 * @file test/cfo/tools/data/tests_data.cpp
 * @author The RECORD Development Team (INRA) and the VLE Development Team
 * See the AUTHORS or Authors.txt file
 */

/*
 * Copyright (C) 2009-2010 INRA
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#define BOOST_TEST_MAIN
#define BOOST_AUTO_TEST_MAIN
#define BOOST_TEST_DYN_LINK
#define BOOST_TEST_MODULE tests_data

#include <stdexcept>
#include <limits>
#include <fstream>
#include <iostream>

#include <boost/test/unit_test.hpp>
#include <boost/test/auto_unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <vle/manager/Manager.hpp>

#include <cfo/tools/data/ReadData.hpp>

const char* cfoDataTest = \
";a;;a\n"
"2;1;1;6\n"
"4;4;8.2;9\n"
"8;5;8;25.0\n"
"1;0;11;1\n";

//Test Data
BOOST_AUTO_TEST_CASE(test_data)
{
    namespace dat = cfo::tools::data;

    //Real Test
    std::vector<std::string> headers;
    headers.push_back("thea");
    std::stringstream stream(cfoDataTest);
    dat::ReadData rd(stream,headers);
    const dat::Data& d = rd.getData();
    BOOST_REQUIRE_CLOSE(d.get(1,2),8.2,10e-4);
    BOOST_REQUIRE_CLOSE(d.get(3,3),1,10e-4);
    BOOST_CHECK((d.getHeader("thea").property(1) == "a"));
}
