/**
 * @file test/cfo/tools/regression/tests_CMAC.cpp
 * @author The RECORD Development Team (INRA) and the VLE Development Team
 * See the AUTHORS or Authors.txt file
 */

/*
 * Copyright (C) 2009-2010 INRA
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#define BOOST_TEST_MAIN
#define BOOST_AUTO_TEST_MAIN
#define BOOST_TEST_DYN_LINK
#define BOOST_TEST_MODULE tests_cfo

#include <stdexcept>
#include <limits>
#include <fstream>
#include <iostream>
#include <boost/test/unit_test.hpp>
#include <boost/test/auto_unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>
#include <boost/lexical_cast.hpp>
#include <vle/value/Map.hpp>
#include <vle/value/Set.hpp>
#include <vle/value/Tuple.hpp>
#include <vle/utils/Rand.hpp>
#include <cfo/tools/regression/CMAC.hpp>
#include <cfo/tools/VleAPIfacilities.hpp>
#include "utils_reg.hpp"

namespace vv = vle::value;
namespace vu = vle::utils;
namespace ct = cfo::tools;
namespace ctr = cfo::tools::regression;

//CMAC regressor
BOOST_AUTO_TEST_CASE(test_CMAC)
{
    vv::Map CMACinit;
    CMACinit.addInt("nb-tiles",6);
    CMACinit.addInt("nb-grids",2);
    vv::Set& bounds = CMACinit.addSet("bounds");
    bounds.addTuple();
    vv::Tuple& p1 = bounds.getTuple(0);
    p1.add(-100);
    p1.add(100);
    bounds.addTuple();
    vv::Tuple& p2 = bounds.getTuple(1);
    p2.add(-100);
    p2.add(100);

    ctr::CMAC reg;
    reg.init(CMACinit);

    vu::Rand r;
    vv::Tuple point(2,0);
    double x1, x2, predicted, trueVal;
    //learn regressor
    for(unsigned int p = 0; p < 100000 ; p++){
        x1 = r.getDouble(-100,100);
        x2 = r.getDouble(-100,100);
        point.at(0) = x1;
        point.at(1) = x2;
        reg.updateWeight(point /* point*/, bohachevsky(x1,x2) /*upValue*/,
            0.3 /*alpha0*/);
    }
    //test regressor
    for(unsigned int p = 0; p < 10 ; p++){
        x1 = r.getDouble(-100,100);
        x2 = r.getDouble(-100,100);
        point.at(0) = x1;
        point.at(1) = x2;
        predicted = reg.predict(point);
        trueVal = bohachevsky(x1,x2);
        std::cout << " predicted:" << predicted << " trueVal:" << trueVal
                << " diff:" <<  predicted - trueVal << std::endl;

    }
    //test the serial-unserial functions of CMAC regressor based on vv::Value
    vv::Value* vreg = reg.toValue();
    std::cout << " vreg  " << *vreg << std::endl;
    ctr::CMAC reg2;
    reg2.fromValue(*vreg);
    vv::Value* vreg2 = reg2.toValue();
    bool verif = ct::VleAPIfacilities::equal(*vreg,*vreg2);
    BOOST_REQUIRE_EQUAL(verif,true);
    delete vreg;
    delete vreg2;
}

